@@ -1,5 +1,6 @@
 from collections import Mapping, Hashable
 from operator import add
+import pytest
 from pyrsistent import pmap, m
 import pickle
 
@@ -28,12 +29,17 @@ def test_initialization_with_one_element():
     assert the_map.a == 2
     assert 'a' in the_map
     
-    assert the_map is the_map.remove('b')
+    assert the_map is the_map.discard('b')
     
     empty_map = the_map.remove('a')
     assert len(empty_map) == 0
     assert 'a' not in empty_map
 
+def test_remove_non_existing_element_raises_key_error():
+    m1 = m(a=1)
+
+    with pytest.raises(KeyError):
+        m1.remove('b')
 
 def test_various_iterations():
     assert set(['a', 'b']) == set(m(a=1, b=2))
@@ -72,9 +78,9 @@ def test_initialization_with_many_elements():
     assert new_map['1601'] == 1601
     
     # Some NOP properties
-    assert new_map.remove('18888') is new_map
+    assert new_map.discard('18888') is new_map
     assert '19999' not in new_map
-    assert new_map['1500'] == 1500  
+    assert new_map['1500'] == 1500
     assert new_map.set('1500', new_map['1500']) is new_map
 
 
@@ -95,6 +101,7 @@ def test_overwrite_existing_element():
     assert len(map2) == 1
     assert map2['a'] == 3
 
+
 def test_supports_hash_and_equals():
     x = m(a=1, b=2, c=3)
     y = m(a=1, b=2, c=3)
@@ -103,8 +110,9 @@ def test_supports_hash_and_equals():
     assert x == y
     assert not (x != y)
 
+
 def test_same_hash_when_content_the_same_but_underlying_vector_size_differs():
-    x = pmap({x: x for x in range(1000)})
+    x = pmap(dict((x, x) for x in range(1000)))
     y = pmap({10: 10, 200: 200, 700: 700})
 
     for z in x:
@@ -114,22 +122,26 @@ def test_same_hash_when_content_the_same_but_underlying_vector_size_differs():
     assert x == y
     assert hash(x) == hash(y)
 
-def test_merge_with_multiple_arguments():
+
+def test_update_with_multiple_arguments():
     # If same value is present in multiple sources, the rightmost is used.
     x = m(a=1, b=2, c=3)    
-    y = x.merge(m(b=4, c=5), {'c': 6})
+    y = x.update(m(b=4, c=5), {'c': 6})
 
     assert y == m(a=1, b=4, c=6)
 
-def test_merge_one_argument():
+
+def test_update_one_argument():
     x = m(a=1)
 
-    assert x.merge(m(b=2)) == m(a=1, b=2)
+    assert x.update(m(b=2)) == m(a=1, b=2)
 
-def test_merge_no_arguments():
+
+def test_update_no_arguments():
     x = m(a=1)
 
-    assert x.merge() is x
+    assert x.update() is x
+
 
 def test_set_in_base_case():
     # Works as set when called with only one key
@@ -137,17 +149,13 @@ def test_set_in_base_case():
     
     assert x.set_in(['a'], 3) == m(a=3, b=2)
 
-def test_set_in_base_case():
-    # Works as set when called with only one key
-    x = m(a=1, b=2)
-    
-    assert x.set_in(['a'], 3) == m(a=3, b=2)
 
 def test_set_in_nested_maps():
     x = m(a=1, b=m(c=3, d=m(e=6, f=7)))
     
     assert x.set_in(['b', 'd', 'e'], 999) == m(a=1, b=m(c=3, d=m(e=999, f=7)))
-    
+
+
 def test_set_in_levels_missing():
     x = m(a=1, b=m(c=3))
     
@@ -195,18 +203,18 @@ def test_hash_collision_is_correctly_resolved():
     assert map3[dummy3] == 3
     
     # Remove elements
-    map4 = map.remove(dummy2)
+    map4 = map.discard(dummy2)
     assert len(map4) == 2
     assert map4[dummy1] == 1
     assert dummy2 not in map4
     assert map4[dummy3] == 3
     
-    assert map.remove(dummy4) == map
+    assert map.discard(dummy4) is map
     
     # Empty map handling
     empty_map = map4.remove(dummy1).remove(dummy3)
     assert len(empty_map) == 0
-    assert empty_map.remove(dummy1) == empty_map
+    assert empty_map.discard(dummy1) is empty_map
     
 
 def test_bitmap_indexed_iteration():
@@ -221,8 +229,8 @@ def test_bitmap_indexed_iteration():
         values.add(v)
     
     assert count == 2
-    assert keys == {'a', 'b'}
-    assert values == {2, 1}
+    assert keys == set(['a', 'b'])
+    assert values == set([2, 1])
 
 
 def test_iteration_with_many_elements():
@@ -258,14 +266,14 @@ def test_empty_truthiness():
     assert m(a=1)
     assert not m()
 
-def test_merge_with():
-    assert m(a=1).merge_with(add, m(a=2, b=4)) == m(a=3, b=4)
-    assert m(a=1).merge_with(lambda l, r: l, m(a=2, b=4)) == m(a=1, b=4)
+def test_update_with():
+    assert m(a=1).update_with(add, m(a=2, b=4)) == m(a=3, b=4)
+    assert m(a=1).update_with(lambda l, r: l, m(a=2, b=4)) == m(a=1, b=4)
 
     def map_add(l, r):
         return dict(list(l.items()) + list(r.items()))
 
-    assert m(a={'c': 3}).merge_with(map_add, m(a={'d': 4})) == m(a={'c': 3, 'd': 4})
+    assert m(a={'c': 3}).update_with(map_add, m(a={'d': 4})) == m(a={'c': 3, 'd': 4})
 
 
 def test_pickling_empty_map():
@@ -273,4 +281,52 @@ def test_pickling_empty_map():
 
 
 def test_pickling_non_empty_vector():
-    assert pickle.loads(pickle.dumps(m(a=1, b=2), -1)) == m(a=1, b=2)
\ No newline at end of file
+    assert pickle.loads(pickle.dumps(m(a=1, b=2), -1)) == m(a=1, b=2)
+
+
+def test_set_with_relocation():
+    x = pmap({'a':1000}, pre_size=1)
+    x = x.set('b', 3000)
+    x = x.set('c', 4000)
+    x = x.set('d', 5000)
+    x = x.set('d', 6000)
+
+    assert len(x) == 4
+    assert x == pmap({'a': 1000, 'b': 3000, 'c': 4000, 'd': 6000})
+
+
+def test_evolver_simple_update():
+    x = m(a=1000, b=2000)
+    e = x.evolver()
+    e['b'] = 3000
+
+    assert e['b'] == 3000
+    assert e.pmap()['b'] == 3000
+    assert x['b'] == 2000
+
+
+def test_evolver_update_with_relocation():
+    x = pmap({'a':1000}, pre_size=1)
+    e = x.evolver()
+    e['b'] = 3000
+    e['c'] = 4000
+    e['d'] = 5000
+    e['d'] = 6000
+
+    assert len(e) == 4
+    assert e.pmap() == pmap({'a': 1000, 'b': 3000, 'c': 4000, 'd': 6000})
+
+
+def test_evolver_remove_element():
+    e = m(a=1000, b=2000).evolver()
+    assert 'a' in e
+
+    del e['a']
+    assert 'a' not in e
+
+
+def test_evolver_remove_element_not_present():
+    e = m(a=1000, b=2000).evolver()
+
+    with pytest.raises(KeyError):
+        del e['c']
